@@ -89,7 +89,7 @@ Rcpp::List allocate_out_list(OGRFeatureDefn *poFDefn, int n_features, bool int64
 	return out;
 }
 
-int to_multi_what(std::vector<OGRGeometry *> gv) {
+OGRwkbGeometryType to_multi_what(std::vector<OGRGeometry *> gv) {
 	bool points = false, multipoints = false,
 		lines = false, multilines = false,
 		polygons = false, multipolygons = false;
@@ -106,7 +106,7 @@ int to_multi_what(std::vector<OGRGeometry *> gv) {
 			case wkbMultiLineString: multilines = true; break;
 			case wkbPolygon: polygons = true; break;
 			case wkbMultiPolygon: multipolygons = true; break;
-			default: return 0; // #nocov
+			default: return wkbUnknown; // #nocov
 		}
 	}
 	int sum = points + multipoints + lines + multilines + polygons + multipolygons;
@@ -119,7 +119,7 @@ int to_multi_what(std::vector<OGRGeometry *> gv) {
 			return wkbMultiPolygon;
 	}
 	// another mix or single type:
-	return 0;
+	return wkbUnknown;
 }
 
 size_t count_features(OGRLayer *poLayer) {
@@ -182,34 +182,37 @@ Rcpp::List CPL_get_layers(Rcpp::CharacterVector datasource, Rcpp::CharacterVecto
 			feature_count(iLayer) = count_features(poLayer);
 	}
 
-	Rcpp::List out(6);
-	out(0) = names;
-	out(1) = geomtype;
-	out(2) = poDS->GetDriverName();
-	out(3) = feature_count;
-	out(4) = field_count;
-	out(5) = layer_crs;
+	Rcpp::List ret = Rcpp::List::create(
+		Rcpp::_["name"] = names,
+		Rcpp::_["geomtype"] = geomtype,
+		Rcpp::_["driver"] = poDS->GetDriverName(),
+		Rcpp::_["features"] = feature_count,
+		Rcpp::_["fields"] = field_count,
+		Rcpp::_["crs"] = layer_crs);
 	GDALClose(poDS); // close & destroys data source
-	out.attr("names") = Rcpp::CharacterVector::create("name", "geomtype", "driver", "features", "fields", "crs");
-	out.attr("class") = Rcpp::CharacterVector::create("sf_layers");
-	return out;
+	return ret;
 }
 
 Rcpp::List sf_from_ogrlayer(OGRLayer *poLayer, bool quiet, bool int64_as_string,
-		Rcpp::NumericVector toTypeUser, Rcpp::CharacterVector fid_column, bool promote_to_multi = true) {
+		Rcpp::NumericVector toTypeUser, Rcpp::CharacterVector fid_column, 
+		bool promote_to_multi = true, int nfeatures = -1) {
 
-	double n_d = (double) poLayer->GetFeatureCount();
-	if (n_d > INT_MAX)
-		Rcpp::stop("Cannot read layer with more than MAX_INT features"); // #nocov
-	if (n_d < 0)
-		n_d = (double) count_features(poLayer);
-	size_t n = (size_t) n_d; // what is List's max length?
+	OGRFeatureDefn *poFDefn = poLayer->GetLayerDefn();
+
+	size_t n = 0;
+	if (nfeatures == -1) {
+		double n_d = (double) poLayer->GetFeatureCount();
+		if (n_d > INT_MAX)
+			Rcpp::stop("Cannot read layer with more than MAX_INT features"); // #nocov
+		if (n_d < 0)
+			n_d = (double) count_features(poLayer);
+		n = (size_t) n_d; // what is List's max length?
+	 } else
+		n = nfeatures;
 
 	std::vector<OGRFeature *> poFeatureV(n); // full archive
 	Rcpp::CharacterVector fids(n);
 
-	OGRFeatureDefn *poFDefn = poLayer->GetLayerDefn();
-
 	std::vector<OGRGeometry *> poGeometryV(n * poFDefn->GetGeomFieldCount());
 	// cycles column wise: 2nd el is 1st geometry, 2nd feature
 
@@ -222,6 +225,10 @@ Rcpp::List sf_from_ogrlayer(OGRLayer *poLayer, bool quiet, bool int64_as_string,
 	bool warn_int64 = false, has_null_geometries = false;
 	OGRFeature *poFeature;
 	while ((poFeature = poLayer->GetNextFeature()) != NULL) {
+		if (i > (n - 1)) {
+			Rcpp::warning("more features available than GetFeatureCount() reported: some records may be failing");
+			break;
+		}
 		// getFID:
 		fids[i] = std::to_string(poFeature->GetFID());
 
@@ -288,14 +295,14 @@ Rcpp::List sf_from_ogrlayer(OGRLayer *poLayer, bool quiet, bool int64_as_string,
 					//  POSIXlt: sec   min  hour  mday   mon  year  wday  yday isdst ...
 					Rcpp::List dtlst =
 						Rcpp::List::create(
-							Rcpp::_["sec"] = (double) Second, 
+							Rcpp::_["sec"] = (double) Second,
 							Rcpp::_["min"] = (int) Minute,
 							Rcpp::_["hour"] = (int) Hour,
 							Rcpp::_["mday"] = (int) Day,
 							Rcpp::_["mon"] = (int) Month - 1,
 							Rcpp::_["year"] = (int) Year - 1900,
-							Rcpp::_["wday"] = NA_INTEGER, 
-							Rcpp::_["yday"] = NA_INTEGER, 
+							Rcpp::_["wday"] = NA_INTEGER,
+							Rcpp::_["yday"] = NA_INTEGER,
 							Rcpp::_["isdst"] = NA_INTEGER,
 							Rcpp::_["zone"] = tzone,
 							Rcpp::_["gmtoff"] = NA_INTEGER);
@@ -420,104 +427,112 @@ Rcpp::List sf_from_ogrlayer(OGRLayer *poLayer, bool quiet, bool int64_as_string,
 		poFeatureV[i] = poFeature;
 		i++;
 	} // all read...
+	if (i < n) { // re-read with preset n: see https://github.com/r-spatial/sf/issues/2248
+		Rcpp::Rcout << "Re-reading with feature count reset from " << n << " to " << i << std::endl;
+		return sf_from_ogrlayer(poLayer, quiet, int64_as_string, toTypeUser, fid_column, promote_to_multi, i);
+	} else { 
 
-	// add feature IDs if needed:
-	if (fid_column.size())
-		out[ poFDefn->GetFieldCount() ] = fids;
+		// add feature IDs if needed:
+		if (fid_column.size())
+			out[ poFDefn->GetFieldCount() ] = fids;
 
-	std::vector<OGRGeometry *> to_be_freed;
-	for (int iGeom = 0; iGeom < poFDefn->GetGeomFieldCount(); iGeom++ ) {
+		std::vector<OGRGeometry *> to_be_freed;
+		for (int iGeom = 0; iGeom < poFDefn->GetGeomFieldCount(); iGeom++ ) {
 
-		std::vector<OGRGeometry *> poGeom(n);
-		for (i = 0; i < n; i++)
-			poGeom[i] = poGeometryV[i + n * iGeom];
+			std::vector<OGRGeometry *> poGeom(n);
+			for (i = 0; i < n; i++)
+				poGeom[i] = poGeometryV[i + n * iGeom];
 
-		int toType = 0, toTypeU = 0;
-		if (toTypeUser.size() == poFDefn->GetGeomFieldCount())
-			toTypeU = toTypeUser[iGeom];
-		else
-			toTypeU = toTypeUser[0];
-		if (promote_to_multi && toTypeU == 0)
-			toType = to_multi_what(poGeom);
-		else
-			toType = toTypeU;
+			OGRwkbGeometryType toType = wkbUnknown, toTypeU = wkbUnknown;
+			if (toTypeUser.size() == poFDefn->GetGeomFieldCount())
+				toTypeU = (OGRwkbGeometryType) toTypeUser[iGeom];
+			else
+				toTypeU = (OGRwkbGeometryType) toTypeUser[0];
+			if (promote_to_multi && toTypeU == wkbUnknown)
+				toType = to_multi_what(poGeom);
+			else
+				toType = toTypeU;
 
-		if (toType != 0) { // coerce to toType:
-			// OGRGeomFieldDefn *poGFDefn = poFDefn->GetGeomFieldDefn(i);
-			for (i = 0; i < poFeatureV.size(); i++) {
-				OGRGeometry *geom = poFeatureV[i]->StealGeometry(iGeom); // transfer ownership
-				if (geom == NULL)
-					geom = OGRGeometryFactory::createGeometry((OGRwkbGeometryType) toType); // #nocov
-				else if ((geom =
-						OGRGeometryFactory::forceTo(geom, (OGRwkbGeometryType) toType, NULL))
-						== NULL)
-					Rcpp::stop("OGRGeometryFactory::forceTo returned NULL"); // #nocov
-				handle_error(poFeatureV[i]->SetGeomFieldDirectly(iGeom, geom));
-				poGeom[i] = poFeatureV[i]->GetGeomFieldRef(iGeom);
-				if (poGeom[i] == NULL)
-					Rcpp::stop("GetGeomFieldRef returned NULL"); // #nocov
-			}
-		} else if (has_null_geometries) {
-			if (! quiet)
-				Rcpp::Rcout << "replacing null geometries with empty geometries" << std::endl; // #nocov
-
-			// replace null's with empty:
-			OGRwkbGeometryType gt = wkbGeometryCollection;
-			for (i = 0; i < poGeom.size(); i++) {
-				if (poGeom[i] != NULL) {
-					gt = poGeom[i]->getGeometryType(); // first non-NULL
-					break;
-				}
-			}
-			for (i = 0; i < poGeom.size(); i++) {
-				if (poGeom[i] == NULL) {
-					poGeom[i] = OGRGeometryFactory::createGeometry(gt);
+			if (toType != 0) { // coerce to toType:
+				// OGRGeomFieldDefn *poGFDefn = poFDefn->GetGeomFieldDefn(i);
+				for (i = 0; i < poFeatureV.size(); i++) {
+					OGRGeometry *geom = poFeatureV[i]->StealGeometry(iGeom); // transfer ownership
+					if (geom == NULL)
+						geom = OGRGeometryFactory::createGeometry((OGRwkbGeometryType) toType); // #nocov
+					else if ((geom =
+							OGRGeometryFactory::forceTo(geom, (OGRwkbGeometryType) toType, NULL))
+							== NULL)
+						Rcpp::stop("OGRGeometryFactory::forceTo returned NULL"); // #nocov
+					handle_error(poFeatureV[i]->SetGeomFieldDirectly(iGeom, geom));
+					poGeom[i] = poFeatureV[i]->GetGeomFieldRef(iGeom);
 					if (poGeom[i] == NULL)
-						Rcpp::stop("createGeometry returned NULL"); // #nocov
-					else
-						to_be_freed.push_back(poGeom[i]);
+						Rcpp::stop("GetGeomFieldRef returned NULL"); // #nocov
+				}
+			} else if (has_null_geometries) {
+				if (! quiet)
+					Rcpp::Rcout << "replacing null geometries with empty geometries" << std::endl; // #nocov
+
+				// replace null's with empty:
+				OGRwkbGeometryType gt = wkbGeometryCollection;
+				for (i = 0; i < poGeom.size(); i++) {
+					if (poGeom[i] != NULL) {
+						gt = poGeom[i]->getGeometryType(); // first non-NULL
+						break;
+					}
+				}
+				for (i = 0; i < poGeom.size(); i++) {
+					if (poGeom[i] == NULL) {
+						poGeom[i] = OGRGeometryFactory::createGeometry(gt);
+						if (poGeom[i] == NULL)
+							Rcpp::stop("createGeometry returned NULL"); // #nocov
+						else
+							to_be_freed.push_back(poGeom[i]);
+					}
 				}
 			}
+			if (! quiet && toTypeU != 0 && n > 0)
+				Rcpp::Rcout << "converted into: " << poGeom[0]->getGeometryName() << std::endl; // #nocov
+			// convert to R:
+			Rcpp::List sfc = sfc_from_ogr(poGeom, false); // don't destroy
+			OGRGeomFieldDefn *fdfn = poFDefn->GetGeomFieldDefn(iGeom);
+			sfc.attr("crs") = create_crs(fdfn->GetSpatialRef()); // overwrite: see #449 for the reason why
+			out[iGeom + poFDefn->GetFieldCount() + fid_column.size()] = sfc;
 		}
-		if (! quiet && toTypeU != 0 && n > 0)
-			Rcpp::Rcout << "converted into: " << poGeom[0]->getGeometryName() << std::endl; // #nocov
-		// convert to R:
-		Rcpp::List sfc = sfc_from_ogr(poGeom, false); // don't destroy
-		OGRGeomFieldDefn *fdfn = poFDefn->GetGeomFieldDefn(iGeom);
-		sfc.attr("crs") = create_crs(fdfn->GetSpatialRef()); // overwrite: see #449 for the reason why
-		out[iGeom + poFDefn->GetFieldCount() + fid_column.size()] = sfc;
+
+		if (warn_int64)
+			Rcpp::Rcout << "Integer64 values larger than " << dbl_max_int64 <<
+				" lost significance after conversion to double;" << std::endl <<
+				"use argument int64_as_string = TRUE to import them lossless, as character" << std::endl;
+
+		// clean up:
+		for (i = 0; i < n; i++)
+			OGRFeature::DestroyFeature(poFeatureV[i]);
+		for (i = 0; i < to_be_freed.size(); i++)
+			OGRGeometryFactory::destroyGeometry(to_be_freed[i]);
+
+		return out;
 	}
-
-	if (warn_int64)
-		Rcpp::Rcout << "Integer64 values larger than " << dbl_max_int64 <<
-			" lost significance after conversion to double;" << std::endl <<
-			"use argument int64_as_string = TRUE to import them lossless, as character" << std::endl;
-
-	// clean up:
-	for (i = 0; i < n; i++)
-		OGRFeature::DestroyFeature(poFeatureV[i]);
-	for (i = 0; i < to_be_freed.size(); i++)
-		OGRGeometryFactory::destroyGeometry(to_be_freed[i]);
-
-	return out;
 }
 
-// [[Rcpp::export]]
-Rcpp::List CPL_read_ogr(Rcpp::CharacterVector datasource, Rcpp::CharacterVector layer,
-		Rcpp::CharacterVector query,
-		Rcpp::CharacterVector options, bool quiet, Rcpp::NumericVector toTypeUser,
-		Rcpp::CharacterVector fid_column_name, Rcpp::CharacterVector drivers,
-		Rcpp::CharacterVector wkt_filter,
-		bool promote_to_multi = true, bool int64_as_string = false,
-		bool dsn_exists = true,
-		bool dsn_isdb = false,
-		int width = 80) {
+static void finalize_dataset_xptr(SEXP dataset_xptr) {
+	GDALDataset *poDS = (GDALDataset*)R_ExternalPtrAddr(dataset_xptr);
+	if (poDS != nullptr) {
+		GDALClose(poDS);
+	}
+}
 
-	// adapted from the OGR tutorial @ www.gdal.org
+Rcpp::List CPL_ogr_layer_setup(Rcpp::CharacterVector datasource, Rcpp::CharacterVector layer,
+		Rcpp::CharacterVector query,
+		Rcpp::CharacterVector options, bool quiet, Rcpp::CharacterVector drivers,
+		Rcpp::CharacterVector wkt_filter,
+		bool dsn_exists,
+		bool dsn_isdb,
+		int width) {
+    // adapted from the OGR tutorial @ www.gdal.org
 	std::vector <char *> open_options = create_options(options, quiet);
 	std::vector <char *> drivers_v = create_options(drivers, quiet);
 	GDALDataset *poDS;
-	poDS = (GDALDataset *) GDALOpenEx( datasource[0], GDAL_OF_VECTOR | GDAL_OF_READONLY, 
+	poDS = (GDALDataset *) GDALOpenEx( datasource[0], GDAL_OF_VECTOR | GDAL_OF_READONLY,
 		drivers.size() ? drivers_v.data() : NULL, open_options.data(), NULL );
 	if( poDS == NULL ) {
 		// could not open dsn
@@ -533,6 +548,11 @@ Rcpp::List CPL_read_ogr(Rcpp::CharacterVector datasource, Rcpp::CharacterVector
 		Rcpp::stop("Cannot open %s; The file doesn't seem to exist.", datasource);
 	}
 
+	// Will close the dataset if some early return/exception prevents GDALClose() from being
+	// called/allows the result to be accessed by the caller.
+	Rcpp::RObject dataset_xptr = R_MakeExternalPtr(poDS, R_NilValue, R_NilValue);
+	R_RegisterCFinalizer(dataset_xptr, &finalize_dataset_xptr);
+
 	if (layer.size() == 0 && Rcpp::CharacterVector::is_na(query[0])) { // no layer specified
 		switch (poDS->GetLayerCount()) {
 			case 0: { // error:
@@ -596,7 +616,7 @@ Rcpp::List CPL_read_ogr(Rcpp::CharacterVector datasource, Rcpp::CharacterVector
 			Rcpp::Rcout << "Reading layer `" << layer[0] << "' from data source ";
 		// if (LENGTH(datasource[0]) > (width - (34 + LENGTH(layer[0]))))
 		Rcpp::String ds(datasource(0));
-		if (layer.size()) { 
+		if (layer.size()) {
 			Rcpp::String la(layer(0));
 			if (strlen(ds.get_cstring()) > (width - (34 + strlen(la.get_cstring()))))
 				Rcpp::Rcout << std::endl << "  ";
@@ -607,13 +627,37 @@ Rcpp::List CPL_read_ogr(Rcpp::CharacterVector datasource, Rcpp::CharacterVector
 		Rcpp::Rcout << "using driver `" << poDS->GetDriverName() << "'" << std::endl;                       // #nocov
 	}
 
+	// Keeps the dataset external pointer alive as long as the layer external pointer is alive
+	Rcpp::RObject layer_xptr = R_MakeExternalPtr(poLayer, R_NilValue, dataset_xptr);
+
+	return Rcpp::List::create(dataset_xptr, layer_xptr);
+}
+
+// [[Rcpp::export]]
+Rcpp::List CPL_read_ogr(Rcpp::CharacterVector datasource, Rcpp::CharacterVector layer,
+		Rcpp::CharacterVector query,
+		Rcpp::CharacterVector options, bool quiet, Rcpp::NumericVector toTypeUser,
+		Rcpp::CharacterVector fid_column_name, Rcpp::CharacterVector drivers,
+		Rcpp::CharacterVector wkt_filter,
+		bool promote_to_multi = true, bool int64_as_string = false,
+		bool dsn_exists = true,
+		bool dsn_isdb = false,
+		int width = 80) {
+    Rcpp::List prep = CPL_ogr_layer_setup(datasource, layer, query, options,
+							quiet,  drivers,
+							wkt_filter,
+							dsn_exists, dsn_isdb, width);
+	OGRDataSource* poDS = (OGRDataSource*)(R_ExternalPtrAddr(prep[0]));
+	OGRLayer* poLayer = (OGRLayer*)R_ExternalPtrAddr(prep[1]);
+
 	Rcpp::List out = sf_from_ogrlayer(poLayer, quiet, int64_as_string, toTypeUser, fid_column_name,
-		promote_to_multi);
+		promote_to_multi, -1);
 
 	// clean up if SQL was used https://www.gdal.org/classGDALDataset.html#ab2c2b105b8f76a279e6a53b9b4a182e0
 	if (! Rcpp::CharacterVector::is_na(query[0]))
 		poDS->ReleaseResultSet(poLayer);
 
 	GDALClose(poDS);
+	R_SetExternalPtrAddr(prep[0], nullptr);
 	return out;
 }
