@@ -1,4 +1,6 @@
+## IGNORE_RDIFF_BEGIN
 library(sf)
+## IGNORE_RDIFF_END
 # aggregate
 pl1 = st_polygon(list(rbind(c(0,0),c(1,0),c(1,1),c(0,0))))
 pl2 = st_polygon(list(rbind(c(0,0),c(1,1),c(0,1),c(0,0))))
@@ -6,7 +8,7 @@ s = st_sf(a = 1:2, geom = st_sfc(pl1, pl2))
 (a = aggregate(s, list(c(1,1)), mean, do_union = FALSE))
 (a = aggregate(s, list(c(1,1)), mean, do_union = TRUE))
 # expect_warning(st_cast(a, "POINT"))
-if (require(sp, quietly = TRUE)) {
+if (suppressPackageStartupMessages(require(sp, quietly = TRUE))) {
  demo(meuse_sf, echo = FALSE, ask = FALSE)
  a = aggregate(meuse_sf, list(meuse_sf$soil), mean)
  print(attributes(a)$agr)
